(***********************************************************************)
(*                                                                     *)
(*                              OCamlGen                               *)
(*                                                                     *)
(*                       OCaml module Generator                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 1997-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: og_misc.ml,v 1.3 2014/01/31 09:51:52 fclement Exp $ *)

exception Fatal_error;;

let fatal_error msg =
  prerr_string ">> Fatal error: "; prerr_endline msg; raise Fatal_error
;;

let hash_table_make n l =
  let t = Hashtbl.create n in
  List.iter (function (k, v) -> Hashtbl.add t k v) l;
  t
;;

let array_map_to_list f v =
  let l = ref [] in
  for i = 0 to Array.length v - 1 do
    l := f v.(i) :: !l
  done;
  List.rev !l
;;

let string_first_char s =
  if String.length s > 0 then s.[0] else invalid_arg "first_char"
;;

let list_map_end f l endl =
  let rec loop = function
  | [] -> endl
  | x :: l -> f x :: loop l in
  loop l
;;

let iter_sep sep f =
  let rec loop = function
  | [] -> ()
  | [ x; ] -> f x
  | x :: l ->
    f x; sep x; loop l in
  loop
;;
