(***********************************************************************)
(*                                                                     *)
(*                              OCamlGen                               *)
(*                                                                     *)
(*                       OCaml module Generator                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 1997-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* Beginning of ocamlprintc generated code *)

let rec print_file ppf = function
  | p_l -> Lib_print.print_List print_phrase ppf p_l

and print_phrase ppf = function
  | Walker_ast.Comment s ->
    Format.fprintf ppf "@[<1>%s@ " "Comment";
    Lib_print.print_quoted_string ppf s; Format.fprintf ppf "%,@]"
  | Walker_ast.Directive d ->
    Format.fprintf ppf "@[<1>%s@ " "Directive"; print_dir ppf d;
    Format.fprintf ppf "%,@]"
  | Walker_ast.Type_definition (r, t) ->
    Format.fprintf ppf "@[<1>%s@ " "Type_definition";
    Format.fprintf ppf "%,@[<1>(@,"; print_rec_flag ppf r;
    Format.fprintf ppf "%,,@ "; print_type_defs ppf t;
    Format.fprintf ppf "%,@,)@]"; Format.fprintf ppf "%,@]"

and print_rec_flag ppf = function
  | Walker_ast.Rec -> Format.fprintf ppf "%s" "Rec"
  | Walker_ast.No_rec -> Format.fprintf ppf "%s" "No_rec"

and print_dir ppf = function
  | Walker_ast.Dir_open s ->
    Format.fprintf ppf "@[<1>%s@ " "Dir_open";
    Lib_print.print_quoted_string ppf s; Format.fprintf ppf "%,@]"
  | Walker_ast.Dir_include s ->
    Format.fprintf ppf "@[<1>%s@ " "Dir_include";
    Lib_print.print_quoted_string ppf s; Format.fprintf ppf "%,@]"

and print_type_defs ppf = function
  | t_l -> Lib_print.print_List print_type_def ppf t_l

and print_type_def ppf = function
  | ((o_t, w), p, t) ->
    Format.fprintf ppf "%,@[<1>(@,"; Format.fprintf ppf "%,@[<1>(@,";
    Og_types_print.print_type_name ppf o_t; Format.fprintf ppf "%,,@ ";
    print_walker_name ppf w; Format.fprintf ppf "%,@,)@]";
    Format.fprintf ppf "%,,@ "; print_poly_arguments ppf p;
    Format.fprintf ppf "%,,@ "; print_type_kind ppf t;
    Format.fprintf ppf "%,@,)@]"

and print_walker_name ppf = function
  | s -> Lib_print.print_quoted_string ppf s

and print_poly_arguments ppf = function
  | n_l -> Lib_print.print_List print_name ppf n_l

and print_name ppf = function | s -> Lib_print.print_quoted_string ppf s

and print_type_kind ppf = function
  | Walker_ast.Record (b, o_v, r) ->
    Format.fprintf ppf "@[<1>%s@ " "Record"; Format.fprintf ppf "%,@[<1>(@,";
    print_buff_argument ppf b; Format.fprintf ppf "%,,@ ";
    Og_types_print.print_visibility ppf o_v; Format.fprintf ppf "%,,@ ";
    print_record_clause ppf r; Format.fprintf ppf "%,@,)@]";
    Format.fprintf ppf "%,@]"
  | Walker_ast.Sum (b, o_v, s) ->
    Format.fprintf ppf "@[<1>%s@ " "Sum"; Format.fprintf ppf "%,@[<1>(@,";
    print_buff_argument ppf b; Format.fprintf ppf "%,,@ ";
    Og_types_print.print_visibility ppf o_v; Format.fprintf ppf "%,,@ ";
    print_sum_clause ppf s; Format.fprintf ppf "%,@,)@]";
    Format.fprintf ppf "%,@]"
  | Walker_ast.Alias (b, o_v, a) ->
    Format.fprintf ppf "@[<1>%s@ " "Alias"; Format.fprintf ppf "%,@[<1>(@,";
    print_buff_argument ppf b; Format.fprintf ppf "%,,@ ";
    Og_types_print.print_visibility ppf o_v; Format.fprintf ppf "%,,@ ";
    print_alias_clause ppf a; Format.fprintf ppf "%,@,)@]";
    Format.fprintf ppf "%,@]"
  | Walker_ast.External (b, e) ->
    Format.fprintf ppf "@[<1>%s@ " "External";
    Format.fprintf ppf "%,@[<1>(@,"; print_buff_argument ppf b;
    Format.fprintf ppf "%,,@ "; print_external_clause ppf e;
    Format.fprintf ppf "%,@,)@]"; Format.fprintf ppf "%,@]"

and print_buff_argument ppf = function
  | p_o -> Lib_print.print_Option print_pattern ppf p_o

and print_record_clause ppf = function
  | r_l -> Lib_print.print_List print_record_field ppf r_l

and print_record_field ppf = function
  | Walker_ast.Record_field (r, f) ->
    Format.fprintf ppf "@[<1>%s@ " "Record_field";
    Format.fprintf ppf "%,@[<1>(@,"; print_record_label ppf r;
    Format.fprintf ppf "%,,@ "; print_function_clause ppf f;
    Format.fprintf ppf "%,@,)@]"; Format.fprintf ppf "%,@]"

and print_record_label ppf = function
  | s -> Lib_print.print_quoted_string ppf s

and print_sum_clause ppf = function
  | s_l -> Lib_print.print_List print_sum_field ppf s_l

and print_sum_field ppf = function
  | Walker_ast.Sum_tag s ->
    Format.fprintf ppf "@[<1>%s@ " "Sum_tag"; print_sum_label ppf s;
    Format.fprintf ppf "%,@]"
  | Walker_ast.Sum_inj (s, f) ->
    Format.fprintf ppf "@[<1>%s@ " "Sum_inj";
    Format.fprintf ppf "%,@[<1>(@,"; print_sum_label ppf s;
    Format.fprintf ppf "%,,@ "; print_function_clause ppf f;
    Format.fprintf ppf "%,@,)@]"; Format.fprintf ppf "%,@]"

and print_sum_label ppf = function | s -> Lib_print.print_quoted_string ppf s

and print_alias_clause ppf = function
  | (f, t_o) ->
    Format.fprintf ppf "%,@[<1>(@,"; print_function_clause ppf f;
    Format.fprintf ppf "%,,@ ";
    Lib_print.print_Option print_type_kind ppf t_o;
    Format.fprintf ppf "%,@,)@]"

and print_external_clause ppf = function | f -> print_function_clause ppf f

and print_function_clause ppf = function
  | Walker_ast.Clause (p, e) ->
    Format.fprintf ppf "@[<1>%s@ " "Clause"; Format.fprintf ppf "%,@[<1>(@,";
    print_pattern ppf p; Format.fprintf ppf "%,,@ "; print_expr ppf e;
    Format.fprintf ppf "%,@,)@]"; Format.fprintf ppf "%,@]"

and print_pattern ppf = function
  | Walker_ast.Pvariable (n, o_t) ->
    Format.fprintf ppf "@[<1>%s@ " "Pvariable";
    Format.fprintf ppf "%,@[<1>(@,"; print_name ppf n;
    Format.fprintf ppf "%,,@ "; Og_types_print.print_type_expr ppf o_t;
    Format.fprintf ppf "%,@,)@]"; Format.fprintf ppf "%,@]"
  | Walker_ast.Ptuple (p_a, o_t) ->
    Format.fprintf ppf "@[<1>%s@ " "Ptuple"; Format.fprintf ppf "%,@[<1>(@,";
    Lib_print.print_Array print_pattern ppf p_a; Format.fprintf ppf "%,,@ ";
    Og_types_print.print_type_expr ppf o_t; Format.fprintf ppf "%,@,)@]";
    Format.fprintf ppf "%,@]"

and print_function_name ppf = function
  | Walker_ast.Fname_arg n ->
    Format.fprintf ppf "@[<1>%s@ " "Fname_arg"; print_name ppf n;
    Format.fprintf ppf "%,@]"
  | Walker_ast.Fname_local n ->
    Format.fprintf ppf "@[<1>%s@ " "Fname_local"; print_name ppf n;
    Format.fprintf ppf "%,@]"
  | Walker_ast.Fname_external n ->
    Format.fprintf ppf "@[<1>%s@ " "Fname_external"; print_name ppf n;
    Format.fprintf ppf "%,@]"

and print_function_expr ppf = function
  | Walker_ast.Fconstant f ->
    Format.fprintf ppf "@[<1>%s@ " "Fconstant"; print_function_name ppf f;
    Format.fprintf ppf "%,@]"
  | Walker_ast.Fapply (f, f_l) ->
    Format.fprintf ppf "@[<1>%s@ " "Fapply"; Format.fprintf ppf "%,@[<1>(@,";
    print_function_name ppf f; Format.fprintf ppf "%,,@ ";
    Lib_print.print_List print_function_expr ppf f_l;
    Format.fprintf ppf "%,@,)@]"; Format.fprintf ppf "%,@]"
  | Walker_ast.Fabs (p_l, e) ->
    Format.fprintf ppf "@[<1>%s@ " "Fabs"; Format.fprintf ppf "%,@[<1>(@,";
    Lib_print.print_List print_pattern ppf p_l; Format.fprintf ppf "%,,@ ";
    print_expr ppf e; Format.fprintf ppf "%,@,)@]"; Format.fprintf ppf "%,@]"
  | Walker_ast.Fparen f ->
    Format.fprintf ppf "@[<1>%s@ " "Fparen"; print_function_expr ppf f;
    Format.fprintf ppf "%,@]"

and print_expr ppf = function
(*  | Walker_ast.Econstant f ->
    Format.fprintf ppf "@[<1>%s@ " "Econstant"; print_function_expr ppf f;
    Format.fprintf ppf "%,@]" *)
  | Walker_ast.Eapply (f, a_l) ->
    Format.fprintf ppf "@[<1>%s@ " "Eapply"; Format.fprintf ppf "%,@[<1>(@,";
    print_function_expr ppf f; Format.fprintf ppf "%,,@ ";
    Lib_print.print_List print_argument ppf a_l;
    Format.fprintf ppf "%,@,)@]"; Format.fprintf ppf "%,@]"
  | Walker_ast.Etuple e_a ->
    Format.fprintf ppf "@[<1>%s@ " "Etuple";
    Lib_print.print_Array print_expr ppf e_a; Format.fprintf ppf "%,@]"

and print_argument ppf = function
  | Walker_ast.Avar p ->
    Format.fprintf ppf "@[<1>%s@ " "Avar"; print_pattern ppf p;
    Format.fprintf ppf "%,@]"
  | Walker_ast.Ainj (f, p) ->
    Format.fprintf ppf "@[<1>%s@ " "Ainj"; Format.fprintf ppf "%,@[<1>(@,";
    print_function_name ppf f; Format.fprintf ppf "%,,@ ";
    print_pattern ppf p; Format.fprintf ppf "%,@,)@]";
    Format.fprintf ppf "%,@]"
;;

(* End of ocamlprintc generated code *)
